'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var Node =
/*#__PURE__*/
function () {
  function Node() {
    var value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
    var isCompleteWord = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

    _classCallCheck(this, Node);

    this.value = value;
    this.children = new Map();
    this.isCompleteWord = isCompleteWord;
  }
  /**
   * @param {char} value
   * @param {Node} node
   */


  _createClass(Node, [{
    key: "setChild",
    value: function setChild(value, node) {
      this.children.set(value, node);
    }
    /**
     * @param  {char} value
     * @return {Node}
     */

  }, {
    key: "getChild",
    value: function getChild(value) {
      return this.children.get(value);
    }
  }]);

  return Node;
}();

var Trie =
/*#__PURE__*/
function () {
  function Trie() {
    _classCallCheck(this, Trie);

    this.root = new Node();
  }
  /**
  * @param {string} word
  */


  _createClass(Trie, [{
    key: "addWord",
    value: function addWord(word) {
      if (!this.isValidWord(word)) {
        throw Error('Invalid Word');
      }

      var current = this.root;
      Array.from(word).forEach(function (letter) {
        if (!current.getChild(letter)) {
          current.setChild(letter, new Node(letter));
        }

        current = current.getChild(letter);
      });
      current.isCompleteWord = true;
    }
    /**
    * @return {Array<string>}
    */

  }, {
    key: "getWords",
    value: function getWords() {
      var words = [];

      var getWords = function getWords(node, wordFragment) {
        node.children.forEach(function (child) {
          if (child.isCompleteWord) {
            words.push(wordFragment + child.value);
          }

          getWords(child, wordFragment + child.value);
        });
      };

      getWords(this.root, '');
      return words;
    }
    /**
     * @param  {string}  word
     * @return {Boolean}
     */

  }, {
    key: "hasWord",
    value: function hasWord(word) {
      var current = this.root;
      Array.from(word).forEach(function (letter) {
        if (!current.getChild(letter)) {
          return false;
        }

        current = current.getChild(letter);
      });
      return current.isCompleteWord;
    }
    /**
     * @param  {string} prefix
     * @return {Node}
     */

  }, {
    key: "find",
    value: function find(prefix) {
      if (!this.isValidWord(prefix)) {
        throw Error('Invalid Word');
      }

      var current = this.root;
      Array.from(prefix).forEach(function (letter) {
        if (!current.getChild(letter)) {
          return null;
        }

        current = current.getChild(letter);
      });
      return current.value === null ? null : current;
    }
    /**
    * @param  {string}  word
    * @return {Boolean}
    */

  }, {
    key: "isValidWord",
    value: function isValidWord(word) {
      return (typeof word === 'string' || word instanceof String) && word.match(/^[A-z]+$/);
    }
  }]);

  return Trie;
}();

exports["default"] = Trie;